@@ -16,14 +16,6 @@ class LinksHandler(BaseHandler):
         self.finish(html)
 
 
-class TwoFAHandler(BaseHandler):
-    async def get(self):
-        user = self.current_user
-        ns = await create_ns(user)
-        html = await self.render_template("2FA.html", **ns)
-        self.finish(html)
-
-
 class ImprintHandler(BaseHandler):
     async def get(self):
         user = self.current_user
@@ -49,7 +41,6 @@ class ToSHandler(BaseHandler):
 
 
 default_handlers.append((r"/links", LinksHandler))
-default_handlers.append((r"/2FA", TwoFAHandler))
 default_handlers.append((r"/imprint", ImprintHandler))
 default_handlers.append((r"/privacy", DPSHandler))
 default_handlers.append((r"/terms", ToSHandler))
