@@ -1,21 +1,35 @@
 import asyncio
 import copy
+import hashlib
+import json
 import logging
 import os
+import shutil
 import socket
 import sys
 import time
 
+import requests
 import yaml
+from dateutil import parser
 from jsonformatter import JsonFormatter
 from jupyterhub.app import app_log
 
+_global_sse = asyncio.Event()
+
+
+def get_global_sse():
+    global _global_sse
+    return _global_sse
+
+
 logged_logger_name = os.environ.get("LOGGER_NAME", "JupyterHub")
 logger_name = "JupyterHub"
 
 _custom_config_cache = {}
 _custom_config_last_update = 0
 _custom_config_file = os.environ.get("CUSTOM_CONFIG_PATH")
+_custom_config_force_init_n_times = 10
 
 _logging_cache = {}
 
@@ -32,22 +46,30 @@ def get_logging_config():
 def get_custom_config():
     global _custom_config_cache
     global _custom_config_last_update
+    global _custom_config_force_init_n_times
     global _logging_cache
 
     # Only update custom_config, if it has changed on disk
     try:
         last_change = os.path.getmtime(_custom_config_file)
-        if last_change > _custom_config_last_update:
+        if (
+            last_change > _custom_config_last_update
+            or _custom_config_force_init_n_times > 0
+        ):
             app_log.debug("Load custom config file.")
             with open(_custom_config_file, "r") as f:
                 ret = yaml.full_load(f)
             _custom_config_last_update = last_change
             _custom_config_cache = ret
 
-            if _custom_config_cache.get("logging", {}) != _logging_cache:
+            if (
+                _custom_config_cache.get("logging", {}) != _logging_cache
+                or _custom_config_force_init_n_times > 0
+            ):
                 _logging_cache = _custom_config_cache.get("logging", {})
                 app_log.debug("Update Logger")
                 update_extra_handlers()
+            _custom_config_force_init_n_times -= 1
 
     except:
         app_log.exception("Could not load custom config file")
@@ -55,7 +77,7 @@ def get_custom_config():
         return _custom_config_cache
 
 
-def filter_custom_config_by_auth_state_keys():
+def filter_custom_config_by_auth_state_keys(user=None):
     custom_config = get_custom_config()
     custom_config_auth_state_keys = custom_config.get(
         "authStateKeys",
@@ -81,6 +103,11 @@ _reservations_cache = {}
 _last_change_reservation = 0
 
 
+def get_last_reservation_change():
+    global _last_change_reservation
+    return _last_change_reservation
+
+
 def get_reservations():
     global _reservations_cache
     return _reservations_cache
@@ -95,11 +122,16 @@ async def update_reservations():
 
     global _reservations_cache
     global _last_change_reservation
-    reservation_key = "/mnt/reservation_key/ssh-privatekey"
+    reservation_key = os.environ.get(
+        "RESERVATION_KEY_PATH", "/mnt/reservation_key/ssh-privatekey"
+    )
     regex_pattern = "([\\S]+)=([\\S]+)"
 
     while True:
         log.info("Run ReservationCheck")
+        prev_reservations_hash = hashlib.sha256(
+            json.dumps(_reservations_cache, sort_keys=True).encode("utf-8")
+        ).hexdigest()
         config = get_custom_config()
         reservation_timeout = config.get("reservationCheck", {}).get("timeout", 3)
         try:
@@ -107,7 +139,8 @@ async def update_reservations():
         except:
             previous_dict = {}
         output_dict = {}
-
+        add_debug_users = config.get("reservationCheck", {}).get("addUsers", [])
+        setAllActive = config.get("reservationCheck", {}).get("setAllActive", False)
         for system, infos in (
             config.get("reservationCheck", {}).get("systems", {}).items()
         ):
@@ -159,10 +192,23 @@ async def update_reservations():
                         output_dict[system].append(
                             copy.deepcopy(reservation_key_values_dict)
                         )
+                        if add_debug_users:
+                            users = output_dict[system][-1]["Users"]
+                            if users:
+                                users += ","
+                            users += ",".join(add_debug_users)
+                            output_dict[system][-1]["Users"] = users
+                        if setAllActive:
+                            output_dict[system][-1]["State"] = "ACTIVE"
 
-        _reservations_cache = output_dict
-        sleep_timer = config.get("reservationCheck", {}).get("interval", 300)
-        _last_change_reservation = time.time()
+        new_reservations_hash = hashlib.sha256(
+            json.dumps(output_dict, sort_keys=True).encode("utf-8")
+        ).hexdigest()
+        if prev_reservations_hash != new_reservations_hash:
+            _reservations_cache = output_dict
+            sleep_timer = config.get("reservationCheck", {}).get("interval", 300)
+            _last_change_reservation = int(time.time())
+            get_global_sse().set()
         await asyncio.sleep(sleep_timer)
 
 
@@ -172,21 +218,25 @@ _incidents_cache = {}
 _last_change_incidents = 0
 
 
-def get_incidents():
+def get_last_incidents_change():
+    global _last_change_incidents
+    return _last_change_incidents
+
+
+def get_incidents(user=None):
     global _incidents_cache
     return _incidents_cache
 
 
 def update_incidents_now():
     # Update incidents every n seconds
-    import shutil
-
-    import requests
-    from dateutil import parser
 
     global _incidents_cache
     global _last_change_incidents
 
+    prev_incident_hash = hashlib.sha256(
+        json.dumps(_incidents_cache, sort_keys=True).encode("utf-8")
+    ).hexdigest()
     static_dir = "/mnt/shared-data/share/jupyterhub/static/images/footer"
 
     log.info("Run IncidentCheck")
@@ -300,8 +350,15 @@ def update_incidents_now():
                 )
     except:
         log.exception("IncidentCheck - Could not check for incidents")
-    _incidents_cache = incidents
-    _last_change_incidents = int(time.time())
+
+    new_incident_hash = hashlib.sha256(
+        json.dumps(incidents, sort_keys=True).encode("utf-8")
+    ).hexdigest()
+    if new_incident_hash != prev_incident_hash:
+        _incidents_cache = incidents
+        _last_change_incidents = int(time.time())
+        get_global_sse().set()
+
     return _incidents_cache
 
 
