@@ -4,6 +4,7 @@ import json
 import os
 import random
 import re
+import time
 from datetime import datetime
 
 from datefinder import find_dates
@@ -15,8 +16,9 @@ from unicorespawner import UnicoreSpawner
 
 from ..apihandler.user_count import get_user_count
 from ..misc import get_custom_config
+from ..misc import get_global_sse
 from ..misc import get_incidents
-from .utils import get_general_spawn_event
+from .utils import recursive_decrypt
 
 
 class CustomJSCSpawner(OutpostSpawner, UnicoreSpawner):
@@ -25,15 +27,26 @@ class CustomJSCSpawner(OutpostSpawner, UnicoreSpawner):
 
 async def extra_labels(spawner, user_options):
     labels = {
-        "hub.jupyter.org/username": spawner.user.name,
-        "hub.jupyter.org/servername": spawner.name,
+        "hub.jupyter.org/username": re.sub(
+            "[^a-zA-Z0-9\_\.\-]", "-", str(spawner.user.name)
+        ),
+        "hub.jupyter.org/servername": re.sub(
+            "[^a-zA-Z0-9\_\.\-]", "-", str(spawner.name)
+        ),
         "component": "singleuser-server",
-        "app": os.environ.get("JUPYTERHUB_APP", "jupyterhub"),
+        "app": re.sub(
+            "[^a-zA-Z0-9\_\.\-]",
+            "-",
+            str(os.environ.get("JUPYTERHUB_APP", "jupyterhub")),
+        ),
     }
-    skip_options = ["name", "userModules", "userdata_path"]
+    skip_options = ["name", "modules", "userdata_path"]
+    skip_options.extend(user_options.get("secret_keys", []))
     for param, value in spawner.user_options.items():
         if param in skip_options:
             continue
+        param = param.replace("@", "_at_")
+        param = re.sub("[^a-zA-Z0-9\_\.\-]", "-", str(param))  # remove forbidden chars
         key = f"hub.jupyter.org/{param}"
         value = re.sub("[^a-zA-Z0-9\_\.\-]", "-", str(value))  # remove forbidden chars
         # check if value is allowed
@@ -53,6 +66,9 @@ async def extra_labels(spawner, user_options):
                 continue
             value = valuez
         labels.update({key: value})
+    for key, value in labels.items():
+        if len(value) > 63:
+            labels[key] = value[:63]
     return labels
 
 
@@ -70,6 +86,7 @@ async def ssh_custom_forward(spawner, port_forward_info):
     local_svc_port = spawner.server.port
     if not local_svc_name:
         local_svc_name = spawner.svc_name
+    local_svc_name = local_svc_name.split(".")[0]
     if "ssh_node" in port_forward_info.keys():
         ssh_node = port_forward_info["ssh_node"]  # ${HOSTNAME_I}
     else:
@@ -161,8 +178,6 @@ async def is_system_allowed(spawner, user_options):
     # To check if the start procedure is allowed, we'll check if
     # - system is not in maintenace (JSC status page)
     # - system is allowed for user's group
-    # - partition is allowed for user's group
-    # - resources (reservation, nodes, runtime, gpu, xserver) is within configured range
     # - [service|system]_limit for user is not exceeded
     user = spawner.user
     named_spawners = list(spawner.user.all_spawners(include_default=False))
@@ -171,14 +186,10 @@ async def is_system_allowed(spawner, user_options):
     auth_state = await user.get_auth_state()
     user_groups = auth_state["groups"]
 
-    dropdown_list = auth_state.get("options_form", {}).get("dropdown_list", {})
-    profile = user_options.get("profile", "")
-    service = profile.split("/")[0]
-    option = profile.split("/")[1]
+    service = user_options["service"]
+    option = user_options["option"]
+    profile = user_options["profile"]
     system = user_options.get("system", "")
-    account = user_options.get("account", "")
-    project = user_options.get("project", "")
-    partition = user_options.get("partition", "")
     # Let's check if the system is in maintenance
     incidents_dict = get_incidents()
     threshold_health = (
@@ -192,113 +203,16 @@ async def is_system_allowed(spawner, user_options):
         raise Exception(
             f"System is currently in maintenance. Current health level: {system_health} (threshold: {threshold_health})"
         )
-    if not partition:
-        # No partition: just check if system is allowed by group resources
-        systems_list = [*dropdown_list.get(option, {})]
-        if not system in systems_list:
-            raise Exception(f"system {system} currently not listed in {systems_list}.")
-    if partition in custom_config.get("systems", {}).get(system, {}).get(
-        "interactivePartitions", []
-    ):
-        # Interactive partition, no need to check for resource values.
-        # Check if partition is allowed in general
-        if (
-            not partition
-            in dropdown_list.get(option, {})
-            .get(system, {})
-            .get(account, {})
-            .get(project, {})
-            .keys()
-        ):
-            raise Exception(
-                f"partition {partition} currently not allowed for {system}."
-            )
-    else:
-        resources = auth_state.get("options_form", {}).get("resources", {})
-        reservation = user_options.get("reservation", "None")
-        if reservation and reservation != "None":
-            reservation_list = (
-                dropdown_list.get(option, {})
-                .get(system, {})
-                .get(account, {})
-                .get(project, {})
-                .get(partition, ["None"])
-            )
-            reservation_names = [
-                x["ReservationName"]
-                for x in reservation_list
-                if type(x) == dict and "ReservationName" in x.keys()
-            ]
-            if reservation not in reservation_names:
-                raise Exception(
-                    f"Selected reservation ( {reservation} ) is not in allowed list ( {reservation_names} )"
-                )
-        nodes = int(user_options.get("nodes", "-1"))
-        if nodes != -1:
-            nodes = int(nodes)
-            nodes_range = (
-                resources.get(option, {})
-                .get(system, {})
-                .get(partition, {})
-                .get("nodes", {})
-                .get("minmax", [-1, -1])
-            )
-            if nodes < nodes_range[0] or nodes > nodes_range[1]:
-                raise Exception(
-                    f"Selected nodes ( {nodes} ) not within allowed range {nodes_range}"
-                )
-        runtime = int(user_options.get("runtime", "-1"))
-        if runtime != -1:
-            runtime = int(runtime)
-            runtime_range = (
-                resources.get(option, {})
-                .get(system, {})
-                .get(partition, {})
-                .get("runtime", {})
-                .get("minmax", [-1, -1])
-            )
-            if runtime < runtime_range[0] or runtime > runtime_range[1]:
-                raise Exception(
-                    f"Selected runtime ( {runtime} ) not within allowed range {runtime_range}"
-                )
-        gpus = int(user_options.get("gpus", "-1"))
-        if gpus != -1:
-            gpus = int(gpus)
-            gpus_range = (
-                resources.get(option, {})
-                .get(system, {})
-                .get(partition, {})
-                .get("gpus", {})
-                .get("minmax", [-1, -1])
-            )
-            if gpus < gpus_range[0] or gpus > gpus_range[1]:
-                raise Exception(
-                    f"Selected gpus ( {gpus} ) not within allowed range {gpus_range}"
-                )
-        xserver = int(user_options.get("xserver", "-1"))
-        if xserver != -1:
-            xserver = int(xserver)
-            xserver_range = (
-                resources.get(option, {})
-                .get(system, {})
-                .get(partition, {})
-                .get("xserver", {})
-                .get("minmax", [-1, -1])
-            )
-            if xserver < xserver_range[0] or xserver > xserver_range[1]:
-                raise Exception(
-                    f"Selected xserver gpu index ( {xserver} ) not within allowed range {xserver_range}"
-                )
 
     service_limit = 0
     for group in user_groups:
         limit = (
-            custom_config.get("groups")
-            .get(group)
-            .get("services")
+            custom_config.get("services", {})
             .get(service, {})
+            .get("options", {})
             .get(option, {})
-            .get("maxPerUser", 0)
+            .get("maxPerUser", {})
+            .get(group, 0)
         )
         if limit > service_limit:
             service_limit = limit
@@ -312,18 +226,17 @@ async def is_system_allowed(spawner, user_options):
                 and spawner.active
             ):
                 current += 1
-        if current >= service_limit:
+        if current > service_limit:
             reason = f'User {user.name} already has the maximum number of {service_limit} servers with configuration "service - {profile}" running simultaneously. One must be stopped before a new server can be created'
             raise Exception(f"Service limit exceeded. {reason}")
 
     system_limit = 0
     for group in user_groups:
         limit = (
-            custom_config.get("groups")
-            .get(group)
-            .get("systems", {})
+            custom_config.get("systems", {})
+            .get(system, {})
             .get("maxPerUser", {})
-            .get(system, 0)
+            .get(group, 0)
         )
         if limit > system_limit:
             system_limit = limit
@@ -337,7 +250,7 @@ async def is_system_allowed(spawner, user_options):
                 and spawner.active
             ):
                 current += 1
-        if current >= system_limit:
+        if current > system_limit:
             reason = f'User {user.name} already has the maximum number of {system_limit} servers with configuration "system - {system}" running simultaneously. One must be stopped before a new server can be created'
             raise Exception(f"Service limit exceeded. {reason}")
 
@@ -355,7 +268,7 @@ async def request_url(spawner, user_options):
         .get("services", "None")
     )
     if url == "None":
-        reason = "JupyterHub configuration does not know backend-service {backend_service} or it has no url configured"
+        reason = f"JupyterHub configuration does not know backend-service {backend_service} (system {system}) or it has no url configured"
         raise Exception(reason)
     if not url.endswith("/"):
         url += "/"
@@ -410,13 +323,31 @@ async def custom_misc(spawner, user_options):
 async def custom_env(spawner, user_options, jupyterhub_api_url):
     env = {}
     custom_config = get_custom_config()
+    user_options_modules = spawner.user_options.get("modules", {})
     for module_type, modules in custom_config.get("userModules", {}).items():
         for module in modules:
             env[f"JUPYTER_MODULE_{module.upper()}_ENABLED"] = int(
-                module in spawner.user_options.get("userModules", [])
+                module in user_options_modules.get(module_type, [])
             )
+
+    for envstring in spawner.user_options.get("envvariables", []):
+        try:
+            key, value = envstring.split("=")
+            env[key] = value
+        except:
+            spawner.log.warning(
+                f"{spawner.name} - env variable {envstring} cannot be used"
+            )
+
     system = user_options.get("system", "")
-    if system in ["JSC-Cloud"]:
+    backend_service = (
+        custom_config.get("systems", {}).get(system, {}).get("backendService", "None")
+    )
+    if (
+        custom_config.get("backendServices", {})
+        .get(backend_service, {})
+        .get("type", "kube")
+    ):
         env[
             "JUPYTERHUB_FLAVORS_UPDATE_URL"
         ] = f"{jupyterhub_api_url.rstrip('/')}/outpostflavors/{system}"
@@ -442,9 +373,46 @@ def custom_port(spawner, user_options):
     return port
 
 
-async def pre_spawn_hook(spawner):
+async def pre_poll_hook(spawner):
     custom_config = get_custom_config()
-    service, version = spawner.user_options.get("profile", "").split("/")
+    system = spawner.user_options.get("system", "")
+    backend_service = (
+        custom_config.get("systems", {}).get(system, {}).get("backendService", "None")
+    )
+    if (
+        custom_config.get("backendServices", {})
+        .get(backend_service, {})
+        .get("pollAccessTokenRequired", False)
+    ):
+        # If we know we need need the access token to perform a valid poll request,
+        # let's avoid the call to the Outpost if it's not helpful anyway.
+        user = spawner.user
+        auth_state = await user.get_auth_state()
+        if not auth_state:
+            return False
+        threshold = 2 * user.authenticator.true_auth_refresh_age
+        now = time.time()
+        rest_time = int(auth_state.get("exp", now)) - now
+        if not auth_state.get("access_token", None) or threshold > rest_time:
+            spawner.log.info(
+                f"{spawner._log_name} - Do not call Outpost for polling (access_token available: {bool(auth_state.get('acccess_token'))}, rest_time: {rest_time}"
+            )
+            return False
+    return True
+
+
+async def pre_spawn_hook(spawner):
+    global_sse = get_global_sse()
+    global_sse.set()
+    custom_config = get_custom_config()
+
+    service = spawner.user_options["service"]
+    for service_displayname, service_options in custom_config.get(
+        "services", {}
+    ).items():
+        if service_options.get("serviceId", "") == service:
+            service = service_displayname
+    option = spawner.user_options["option"]
     system = spawner.user_options.get("system", "")
     default_version = (
         custom_config.get("systems", {})
@@ -458,7 +426,7 @@ async def pre_spawn_hook(spawner):
             custom_config.get("services", {}).get(service, {}).get("defaultOption", "")
         )
     try:
-        version_number = float(version)
+        version_number = float(option)
         default_number = float(default_version)
         if version_number < default_number:
             now = datetime.now().strftime("%Y-%m-%d %H:%M:%S.%f")[:-3]
@@ -466,35 +434,53 @@ async def pre_spawn_hook(spawner):
                 "failed": False,
                 "ready": False,
                 "progress": 15,
-                "html_message": f'<details><summary>{now}: <span style="color:darkorange;">JupyterLab {version} is deprecated - you might want to switch to {default_version}</span></summary><p>JupyterLab versions < {default_version} are no longer actively supported even though they will be operational.</p></details>',
+                "html_message": f'<details><summary>{now}: <span style="color:darkorange;">JupyterLab {option} is deprecated - you might want to switch to {default_version}</span></summary><p>JupyterLab versions < {default_version} are no longer actively supported even though they will be operational.</p></details>',
             }
-            spawner.latest_events.append(deprecated_warning_event)
+            spawner.events.append(deprecated_warning_event)
     except ValueError:
         pass
 
 
-def post_spawn_request_hook(spawner, resp_json):
+async def post_spawn_request_hook(spawner, resp_json):
     db = inspect(spawner.user.orm_user).session
     get_user_count(db, force=True)
-    spawn_event = get_general_spawn_event()
-    spawn_event.set()
+    global_sse = get_global_sse()
+    global_sse.set()
 
 
-def post_stop_hook(spawner):
+async def post_stop_hook(spawner):
     db = inspect(spawner.user.orm_user).session
     get_user_count(db, force=True)
-    spawn_event = get_general_spawn_event()
-    spawn_event.set()
+    global_sse = get_global_sse()
+    global_sse.set()
 
 
 async def progress_ready_hook(spawner, ready_event):
-    ready_msg = f"Service {spawner.name} started."
-    now = datetime.now().strftime("%Y-%m-%d %H:%M:%S.%f")[:-3]
     url = ready_event["url"]
+    ready_msg = f"Service {spawner.name} started."
+    detail_msg = f'You will be redirected to <a href="{url}" target="_blank">{url}</a>'
+
+    now = datetime.now().strftime("%Y-%m-%d %H:%M:%S.%f")[:-3]
     ready_event[
         "html_message"
-    ] = f'<details><summary>{now}: {ready_msg}</summary>You will be redirected to <a href="{url}">{url}</a></details>'
-    spawner.latest_events.append(ready_event)
+    ] = f"<details><summary>{now}: {ready_msg}</summary>{detail_msg}</a></details>"
+
+    # Prevent multpiple logs of the same message with different timestamps
+    append_event = True
+    try:
+        for event in getattr(spawner, "events", []):
+            if ready_msg in event.get("html_message", "") and detail_msg in event.get(
+                "html_message", ""
+            ):
+                append_event = False
+                break
+    except:
+        spawner.log.exception(
+            f"{spawner._log_name} - Could not check if event is already in events"
+        )
+
+    if append_event:
+        spawner.events.append(ready_event)
     return ready_event
 
 
@@ -529,21 +515,13 @@ def poll_interval(spawner, user_options):
 
 async def stop_event(spawner):
     now = datetime.now().strftime("%Y-%m-%d %H:%M:%S.%f")[:-3]
-    system = spawner.user_options.get("system")
     event = {
         "failed": True,
         "ready": False,
         "progress": 100,
         "message": "",
+        "html_message": f"<details><summary>{now}: JupyterLab stopped.</summary></details>",
     }
-    if system:
-        event[
-            "html_message"
-        ] = f"<details><summary>{now}: JupyterLab stopped.</summary>Notebook server could not be started on {system}.</details>"
-    else:
-        event[
-            "html_message"
-        ] = f"<details><summary>{now}: JupyterLab stopped.</summary>Notebook server could not be started.</details>"
     return event
 
 
@@ -584,8 +562,15 @@ def filter_events(spawner, event):
 
 
 async def custom_user_options(spawner, user_options):
-    if "dockerregistry" in user_options.keys():
-        ret = copy.deepcopy(user_options)
-        ret["dockerregistry"] = decrypt(user_options["dockerregistry"])
-        return ret
-    return user_options
+    ret = copy.deepcopy(user_options)
+    log_user_options = copy.deepcopy(user_options)
+    ret = await recursive_decrypt(user_options.get("secret_keys", []), user_options)
+    log_extras = {
+        "userid": spawner.user.id,
+        "username": spawner.user.name,
+        "id": spawner.name,
+        "startid": spawner.start_id,
+        "user_options": log_user_options,
+    }
+    spawner.log.info(f"{spawner._log_name} - Start service", extra=log_extras)
+    return ret
